use strict;
use warnings;
use Test::More tests => 6;

use_ok 'AnyEvent::Finger';
use_ok 'AnyEvent::Finger::Server';
use_ok 'AnyEvent::Finger::Client';
use_ok 'AnyEvent::Finger::Request';
use_ok 'AnyEvent::Finger::Response';
use_ok 'AnyEvent::Finger::Transaction';
